import os
import logging
import formencode
import traceback
import pylons
import paste
import datetime

from pylons import session, g, c
from waskaweb.lib.validators import EvaluationFormValidator
from waskaweb.lib.base import *
from waskaweb.lib.db import db
from waskaweb.lib.helpers import format_date, get_phasesuccessors 
from libmpuls.evaluation.config import EvaluationConfig
from libmpuls.evaluation.evaluation import EvaluationSet
from libmpuls.evaluation.export import EvaluationExportXML, EvaluationExportCSV, EvaluationExportHTML

log = logging.getLogger(__name__)

def get_configfile(id):
    for enabled_eval in g.mpuls_config.get('evaluations', 'enabled'):
        if enabled_eval.get('id') == str(id):
            filename = enabled_eval.get('config')
            path = os.path.join(
                pylons.config.get('pylons.paths').get('static_files'), 
                'xml',
                filename)
            return path
    return None

def get_search_options(soptions=None, id=None):
    options = {}
    options['id'] = id
    if soptions:
        options['start_date'] = soptions.get('start_date') or datetime.date(2009,9,1)
        options['end_date'] = soptions.get('end_date') or datetime.date.today() 

        phase = [int(p) for p in soptions.get('phase')]
        if len(phase) == 1 and -1 in phase:
            options['start_date_field'] = g.mpuls_config.get('evaluations', 'default-start-date-field')
            options['end_date_field'] = g.mpuls_config.get('evaluations', 'default-end-date-field')
        else:
            options['start_date_field'] = g.mpuls_config.get('agency-fields-mapping', 'date_phase%s_start' % min(phase))
            options['end_date_field'] = g.mpuls_config.get('agency-fields-mapping', 'date_phase%s_end' % max(phase))

        options['phase'] = max(phase)
        options['sql'] =  soptions.get('sql') or None
    else:
        # set default evaluation options.
        options['phase'] = 1
        options['start_date'] = g.mpuls_config.get('search', 'default-start-date')
        options['end_date'] = g.mpuls_config.get('search', 'default-end-date')
        options['start_date_field'] = g.mpuls_config.get('evaluations', 'default-start-date-field')
        options['end_date_field'] = g.mpuls_config.get('evaluations', 'default-end-date-field')

    options['typelist'] = c.evalconfig.get_evaluations()

    phases = []
    if soptions:
        phases = soptions.get('phase', [])
    else:
        for p in g.mpuls_config.get('search', 'phases'):
            pt = [t for t in p.values() if int(t) >= 0]
            phases.extend(pt)
    options['phase'] =  phases

    # convert dates to locale
    options['start_date'] = format_date(options['start_date']) 
    options['end_date']   = format_date(options['end_date']) 

    # add (overwirte) sqlsearch from caseoverview
    options['sqlwhere'] = session.get('caseoverview_search') 

    return options

class EvaluateController(BaseController):

    def index(self):
        return self.evaluate()

    def resetFilter(self, id):
        if session.has_key('caseoverview_search'):
            del session['caseoverview_search']
            session.save()
        return self.evaluate(id)

    def evaluate(self, id=None):
        conn, cur = None, None
        c.eval_id = id
        try:
            try:
                config_file = get_configfile(id)
                conn = db.getConnection()
                c.evalconfig   = EvaluationConfig(
                                config_file,
                                conn,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None)
            except:
                print 'Error: Evaluation failed %s' % traceback.print_exc()
                log.error(_('Error: Evaluation failed'))
        finally:
            db.recycleConnection(conn, cur)

        c.evaloptions = get_search_options(session.get('evaluation.options'), id)

        # If user selects adele-evaluation render page with disabled configuration elements. 
        # Change default params
        if id == '0':
            c.evaloptions['phase'] = '2'
            c.evaloptions['start_date_field'] = ['datum_cm_start']
            c.evaloptions['end_date_field'] = ['datum_cm_end']
            form = render('/evaluation/evaluate_adele.mako')
        else:
            form = render('/evaluation/evaluate.mako')
        return formencode.htmlfill.render(unicode(form, 'utf-8'), \
                defaults=c.evaloptions, \
                errors={}, \
                auto_insert_errors=False)

    def _get_evalparams(self, form_result):
        params = {}
        params['id'] = form_result['id']
        params['start_date'] = str(form_result['start_date'])
        params['end_date'] = str(form_result['end_date'])
        params['typelist'] = form_result['typelist']
        params['phase']    = form_result['phase']
        params['ending']    = form_result['type_ending']

        # Dates
        # Build timeframes bases on selected phases.
        dates = []
        all_phases = []
        tdates = []
        phase = params.get('phase')
        dates.append('( ')
        if params.get('start_date') != 'None' and params.get('end_date') != 'None':
            if phase:
                for p in phase:
                    if int(p) >= 0: # phase is unknown -> has no start and endphase
                        sdf = g.mpuls_config.get('agency-fields-mapping', 'date_phase%s_start' % p)
                        edf = g.mpuls_config.get('agency-fields-mapping', 'date_phase%s_end' % p) 
                        suc = get_phasesuccessors(p)
                        all_phases.append(int(p))
                        tdates.append("('%s'::date <= %s AND '%s'::date >= %s AND phase IN (%s) )" % (params.get('start_date'), edf, params.get('end_date'), sdf, ",".join(["%s" % s for s in suc])))
                    else:
                        tdates.append('phase IN (-1)')
                dates.append(" OR ".join(tdates))
            else:
                dates.append("FALSE")
        else:
            dates.append("TRUE")
        dates.append(' )')

        # Type of ending
        ending = " TRUE " 
        endingdate = " TRUE "
        typeendfield = g.mpuls_config.get('agency-fields-mapping', 'typeending')
        if params['ending'] and params.get('start_date') != 'None' and params.get('end_date') != 'None':
            enddatefield = g.mpuls_config.get('agency-fields-mapping', 'date_phase3_end')
            endingdate = "( '%s'::date <= %s AND %s <= '%s'::date )" % (params['start_date'], enddatefield, enddatefield, params['end_date'])
            ending = "%s in ( %s )" % (typeendfield ,", ".join(['%s' % e for e in params['ending']]))


        # Add search criteria from case_overview
        search_overview = session.get('caseoverview_search')
        if not search_overview:
            search_overview = "TRUE"

        params['sql'] = "SELECT %%(fields)s from master_tbl_eval_total_view m WHERE %s AND %s AND %s AND %s" % ("".join(dates), ending, endingdate, search_overview)

        #Datefields
        if all_phases:
            params['start_date_field'] = g.mpuls_config.get('agency-fields-mapping', 'date_phase%s_start' % min(all_phases))
            params['end_date_field'] = g.mpuls_config.get('agency-fields-mapping', 'date_phase%s_end' % max(all_phases))
        else:
            params['start_date_field'] = g.mpuls_config.get('evaluations', 'default-start-date-field')
            params['end_date_field'] =  g.mpuls_config.get('evaluations', 'default-end-date-field')
        return params


    def evaluateAction(self):
            '''Return an HTML file containing the result of one or more evaluations.'''
            params = formencode.variabledecode.variable_decode(request.params)
            #params = request.params
            validator    = EvaluationFormValidator() 
            form_result  = {}
            form_errors  = {}

            # Check values
            conn, cur = None, None
            conn = db.getConnection()
            id = int(params['id'])
            c.evalconfig   = EvaluationConfig(
                            get_configfile(id),
                            conn,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None)
            try:
                form_result  = validator.to_python(params)
            except formencode.Invalid, error:
                form_result = error.value
                form_errors = error.error_dict or {}
                c.evaloptions = get_search_options(session.get('evaluation.options'), id)
                form = render('/evaluation/evaluate.mako')
                return formencode.htmlfill.render(unicode(form, 'utf-8'), \
                        defaults=form_result, \
                        errors=form_errors, \
                        auto_insert_errors=False)

            # Build evaluation
            eval_params = self._get_evalparams(form_result)
            try:
                try:
                    evalconfig   = EvaluationConfig(
                                    get_configfile(form_result['id']),
                                    conn,
                                    eval_params['start_date'], 
                                    eval_params['end_date'],
                                    eval_params['start_date_field'], 
                                    eval_params['end_date_field'], 
                                    None,
                                    eval_params['sql'], 
                                    eval_params['typelist'],
                                    )
                    evalset      = EvaluationSet(evalconfig, True)
                    evalset.evaluate()
                    c.result = evalset.export(EvaluationExportHTML(show_percent=form_result['show_percent']))
                    session['evaluation.params'] = eval_params
                    session.save()
                    form = render('/evaluation/result.mako')
                    return formencode.htmlfill.render(unicode(form, 'utf-8'), \
                        defaults=form_result, \
                        errors=form_errors, \
                        auto_insert_errors=False)
                except:
                    log.error(_('Error: Evaluation failed'))
                    log.error(traceback.print_exc())
            finally:
                db.recycleConnection(conn, cur)

    def exportXML(self):
        '''Return an XML file containing the result of one or more evaluations.'''
        #Uncommented default way to return file attachments as IE dies not cope with it. Using hackish way with building FileApps 
        #response.headers['Content-Type'] = 'application/xml; charset=utf8'
        response.headers['Content-Disposition'] = 'attachment; filename=evaluation-export.xml'
        form_result = session.get('evaluation.params')
        try:
            try:
                conn, cur = db.getConnection(), None
                evalconfig   = EvaluationConfig(
                                get_configfile(form_result['id']),
                                conn,
                                form_result['start_date'], 
                                form_result['end_date'],
                                form_result['start_date_field'], 
                                form_result['end_date_field'], 
                                None,
                                form_result['sql'], 
                                form_result['typelist'])
                evalset      = EvaluationSet(evalconfig)
                evalset.evaluate()
                # DataApp need string as input
                content = evalset.export(EvaluationExportXML()).encode('latin1')
                data = paste.fileapp.DataApp(
                    content,
                    content_type='application/xml',
                    content_encoding='UTF-8',
                    cache_control='must-revalidate',
                    pragma='public')
                return data(request.environ, self.start_response)
            except:
                log.error(_('Error: Evaluation failed'))
                log.error(traceback.print_exc())
        finally:
            db.recycleConnection(conn, cur)


    def exportCSV(self):
        '''Return an CSV file containing the result of one or more evaluations.'''
        #Uncommented default way to return file attachments as IE dies not cope with it. Using hackish way with building FileApps 
        #response.headers['Content-Type'] = 'application/xml; charset=utf8'
        response.headers['Content-Disposition'] = 'attachment; filename=evaluation-export.csv'
        form_result = session.get('evaluation.params')
        try:
            try:
                conn, cur = db.getConnection(), None
                evalconfig   = EvaluationConfig(
                                get_configfile(form_result['id']),
                                conn,
                                form_result['start_date'], 
                                form_result['end_date'],
                                form_result['start_date_field'], 
                                form_result['end_date_field'], 
                                None,
                                form_result['sql'], 
                                form_result['typelist'])
                evalset      = EvaluationSet(evalconfig)
                evalset.evaluate()
                # DataApp need string as input
                content = evalset.export(EvaluationExportCSV()).encode('latin1')
                data = paste.fileapp.DataApp(
                    content,
                    content_type='application/csv',
                    content_encoding='UTF-8',
                    cache_control='must-revalidate',
                    pragma='public')
                return data(request.environ, self.start_response)
            except:
                log.error(_('Error: Evaluation failed'))
                log.error(traceback.print_exc())
        finally:
            db.recycleConnection(conn, cur)
