# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
#

import re

#import waskaweb.model.data as data
import waskaweb

from datetime import date

from sys import maxint

class SematicError(Exception):

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)

class ValueChecker:
    def checkAndConvert(self, value, nc):
        return None

class Converter:
    def convert(self, value, nc):
        return None

class TextChecker:

    def convert(self, value, nc):
        return self.checkAndConvert(value, nc)

    def checkAndConvert(self, value, nc):
        
        ml = nc.getMaxLength()
        if ml:
            try:
                value = value[:int(ml)]
                try:
                    return unicode(str(value), 'utf-8')
                except (TypeError, UnicodeError):
                    return value
            except ValueError:
                raise SematicError("Invalid max length value")
        return value

class TextAreaChecker:

    def convert(self, value, nc):
        return self.checkAndConvert(value, nc)

    def checkAndConvert(self, value, nc):
        try:
            return unicode(str(value), 'utf-8')
        except (TypeError, UnicodeError):
            pass
        return value

class IntChecker(ValueChecker):

    def convert(self, value, nc):
        return self.checkAndConvert(value, nc)

    def checkAndConvert(self, value, nc):
        value = value.strip()
        try:
            value = int(float(value))
        except ValueError:
            raise SematicError("'%s' ist keine ganze Zahl." % value)
        try:
            minv = nc.getMinValue()
            maxv = nc.getMaxValue()

            if isinstance(minv, (type(u""), type(""))):
                if len(minv): minv = int(minv)
                else:         minv = -maxint-1
            else:
                minv = int(minv)

            if isinstance(maxv, (type(""), type(u""))):
                if len(maxv): maxv = int(maxv)
                else:         maxv = maxint

            minv, maxv = min(minv, maxv), max(minv, maxv)

            if value < minv:
                raise SematicError(
                    u"%d ist kleiner als der zulässige Minimalwert %d." % (value, minv))

            if value > maxv:
                raise SematicError(
                    u"%d ist größer als der zulässige Maximalwert %d." % (value, maxv))
        except ValueError:
            raise SematicError(
                "Check constraints for '%s'" % nc.getName())
        return value

class DateChecker(ValueChecker):

    DATE         = re.compile(r"([0-9]{1,2})\.([0-9]{1,2})\.([0-9]{2,4})")
    DATE_IDX     = (1, 2, 3)
    DATE_ISO     = re.compile(r"([0-9]{2,4})-([0-9]{1,2})-([0-9]{1,2})")
    DATE_ISO_IDX = (3, 2, 1)

    def convert(self, value, nc):
        return self.checkAndConvert(value, nc)

    def checkAndConvert(self, value, nc):
        value = value.strip()

        for reg, idx in ((DateChecker.DATE, DateChecker.DATE_IDX), (DateChecker.DATE_ISO, DateChecker.DATE_ISO_IDX)):
            m = reg.match(value)
            if not m: continue
            day   = int(m.group(idx[0]))
            month = int(m.group(idx[1]))
            year  = int(m.group(idx[2]))

            if year < 100:
                if year < 70: year += 2000
                else:         year += 1900

            try: 
                return date(year, month, day)
            except ValueError:
                raise SematicError(u"'%s' ist kein gültiges Datum." % value)

        raise SematicError("'%s' ist kein Datum." % value)

class ChoiceChecker(ValueChecker):

    def checkAndConvert(self, value, nc):

        value = value.strip()
        for c in nc.children:
            if isinstance(c, waskaweb.model.data.BoolLeaf):
                v = c.getValue()
                if v == value: 
                    try:
                        return int(v)
                    except ValueError:
                        return v
            elif isinstance(c, waskaweb.model.data.ExternalChoiceListLeaf):
                cc = c.getChildren()
                if cc:
                    for i in cc:
                        if isinstance(i, waskaweb.model.data.BoolLeaf):
                            v = i.getValue()
                            if v == value:
                                try:
                                    return int(v)
                                except ValueError:
                                    return v
            elif isinstance(c, waskaweb.model.data.GroupNode):
                cc = c.children
                if cc:
                    for i in cc:
                        if isinstance(i, waskaweb.model.data.BoolLeaf):
                            v = i.getValue()
                            if v == value:
                                try:
                                    return int(v)
                                except ValueError:
                                    return v

        raise SematicError("'%s' is not a valid value for '%s'" % (
            value, nc.getName()))

def _checkBool(nc, value):
    for m in ("getDescription", "getAlternative"):
        v = getattr(nc, m)()
        if v is None: continue
        v = v.strip().lower()
        if v == value: 
            try:
                return int(nc.getValue().strip())
            except ValueError:
                return v
    return None

class ChoiceConverter(Converter):
    
    def convert(self, value, nc):

        value = value.strip().lower()
        for c in nc.children:
            if isinstance(c, waskaweb.model.data.BoolLeaf):
                v = _checkBool(c, value)
                if not v is None: return v
            elif isinstance(c, waskaweb.model.data.ExternalChoiceListLeaf):
                cc = c.getChildren()
                if cc:
                    for i in cc:
                        if isinstance(i, waskaweb.model.data.BoolLeaf):
                            v = _checkBool(i, value)
                            if not v is None: return v

        raise SematicError(u"'%s' ist kein valider Wert für '%s'" % (
            value, nc.getName()))

class RadioChecker(ValueChecker):

    def checkAndConvert(self, value, nc):
        value = value.strip()
        for c in nc.children:
            if isinstance(c, waskaweb.model.data.BoolLeaf):
                v = c.getValue()
                if v == value: return v
        raise SematicError("'%s' is not a valid value for '%s'" % (
            value, nc.getName()))

class RadioConverter(Converter):

    def convert(self, value, nc):
        value = value.strip()
        for c in nc.children:
            if isinstance(c, waskaweb.model.data.BoolLeaf):
                v = c.getDescription().strip().lower()
                if v == value:
                    return c.getValue()
        raise SematicError("'%s' is not a valid value for '%s'" % (
            value, nc.getName()))

class BoolChecker(ValueChecker):

    def checkAndConvert(self, value, nc):
        value = value.strip()
        return len(value) and 1 or 0 

class BoolConverter(Converter):

    def convert(self, value, nc):
        value = value.strip().lower()
        if not value: return None
        return (value == "ja" or value == '1') and 1 or 0 

TYPES = {
    waskaweb.model.data.IntLeaf :     IntChecker(),
    waskaweb.model.data.DateLeaf:     DateChecker(),
    waskaweb.model.data.TextLeaf:     TextChecker(),
    waskaweb.model.data.TextAreaLeaf: TextAreaChecker(),
    waskaweb.model.data.ChoiceNode:   ChoiceChecker(),
    waskaweb.model.data.BoolLeaf:     BoolChecker(),
    waskaweb.model.data.RadioNode:    RadioChecker()
}

CONVERTERS = {
    waskaweb.model.data.IntLeaf :     IntChecker(),
    waskaweb.model.data.DateLeaf:     DateChecker(),
    waskaweb.model.data.TextLeaf:     TextChecker(),
    waskaweb.model.data.TextAreaLeaf: TextAreaChecker(),
    waskaweb.model.data.ChoiceNode:   ChoiceConverter(),
    waskaweb.model.data.BoolLeaf:     BoolConverter(),
    waskaweb.model.data.RadioNode:    RadioConverter()
}

def checkAndConvert(nc, value):
    try:
        checker = TYPES[nc.__class__]
    except KeyError:
        raise SematicError("no type check found for '%s'"
            % str(nc.__class__))
    return checker.checkAndConvert(value, nc)

def convertFromDescription(nc, value):
    try:
        converter = CONVERTERS[nc.__class__]
    except KeyError:
        raise SematicError("no converter found for '%s'"
            % str(nc.__class__))
    return converter.convert(value, nc)
        
# vim:set ts=4 sw=4 si et sta sts=4:
