# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
#

from waskaweb.lib.db import db

from string import Template
import waskaweb.lib.filters as F

GET_STATEMENT_FLAG_SQL = """SELECT einverstaendniserklaerung from page_einverstaendnis_view WHERE id = %(id)s"""
SET_STATEMENT_FLAG_SQL = """SELECT set_ee_status(%(id)s, %(value)s)"""
class Statement:
    def __init__(self, id=None):
        self.case_id = id
        self.content = ""
    
    def _substitute(self, client, agency):
        return ''

    def _fetchData(self):
        pass

    def getContent(self):
        return self.content

    def fillout (self, client, agency, **kwargs):
        return self._substitute(client, agency)

    def decline(self):
        pass

    def sign(self):
        pass

    def keepUndefined(self):
        pass

    def isSigned(self):
        return False

class DiscretionStatement(Statement):
    def __init__(self, id=None):
        Statement.__init__(self, id)

    def fillout(self, client, agency, **kwargs):
        tmpl = agency.getDiscretionStatement()
        self.template = Template(tmpl)
        self.content = self._substitute(client, agency) 
        return self.content 

    def _substitute(self, client, agency):
        # Substitution dictionary
        s = {
                'VORNAME': F.NA(client.first_name),
                'NACHNAME': F.NA(client.last_name),
                'KA_NAME': F.NA(agency.getName()),
            }
        result = self.template.safe_substitute(s)
        return result

class PrivacyStatement(Statement):
    def __init__(self, id=None):
        Statement.__init__(self, id)

    def fillout(self, client, agency, **kwargs):
        tmpl = agency.getPrivacyStatement()
        self.template = Template(tmpl)
        self.content = self._substitute(client, agency) 
        return self.content 

    def _substitute(self, client, agency):
        # Substitution dictionary
        s = {
                'VORNAME': F.NA(client.first_name),
                'NACHNAME': F.NA(client.last_name),
                'PLZ': F.NA(client.plz),
                'STRASSE': F.NA(client.street),
                'ORT': F.NA(client.city),
                'KA_NAME': F.NA(agency.getName()),
                'KA_SPEICHERDAUER': F.NA(agency.getMaxSavetime())
            }
        result = self.template.safe_substitute(s)
        return result 

    def sign(self):
        conn, cur = None, None
        try:
            conn = db.getConnection()
            cur = conn.cursor()
            cur.execute(SET_STATEMENT_FLAG_SQL, {'id': self.case_id, 'value': 1})
            conn.commit()
        finally:
            db.recycleConnection(conn, cur)

    def keepUndefined(self):
        conn, cur = None, None
        try:
            conn = db.getConnection()
            cur = conn.cursor()
            cur.execute(SET_STATEMENT_FLAG_SQL, {'id': self.case_id, 'value': -1})
            conn.commit()
        finally:
            db.recycleConnection(conn, cur)

    def decline(self):
        conn, cur = None, None
        try:
            conn = db.getConnection()
            cur = conn.cursor()
            cur.execute(SET_STATEMENT_FLAG_SQL, {'id': self.case_id, 'value': 0})
            conn.commit()
        finally:
            db.recycleConnection(conn, cur)

    def isSigned(self):
        '''Returns True is teh statement is signed. False if the statement is
        _not_ signed and will not be signed in the future. None is returned if
        the statement is not signed yet, but might be signed in the future'''
        conn, cur = None, None
        try:
            conn = db.getConnection()
            cur = conn.cursor()
            cur.execute(GET_STATEMENT_FLAG_SQL, {'id': self.case_id})
            signed = cur.fetchone()[0]
            if signed == 1: return True
            elif signed == 0: return False
            elif signed == -1: return None
        finally:
            db.recycleConnection(conn, cur)

class AidPlanStatement(Statement):
    def __init__(self, id=None):
        Statement.__init__(self, id)

    def fillout(self, client, agency, **kwargs):
        list = self._buildAidList(kwargs.get('rg_list', []))
        tmpl = agency.getAidPlanStatement()
        self.template = Template(tmpl)
        self.content = self._substitute(client, agency, list) 
        return self.content 

    def _buildAidList(self, rg_list):
        out = []
        out.append(u'<table width="100%">\n<tr>')
        out.append(u'<th width="100">Von</th>')
        out.append(u'<th width="100">Bis</th>')
        out.append(u'<th>Institution</th>')
        out.append(u'<th>Art der Unterstützung</th>')
        out.append(u'<th>Angebotsform</th>')
        out.append(u"</tr>")
        if len(rg_list) <= 0:
            out.append(u'<tr><td colspan="4">Keine Angebote gefunden</td></tr>')
        else:
            for rg in rg_list:
                out.append(u"""<tr style="border:0">""")
                out.append(u"""<td style="border:0">%s</td>""" % F.NA(rg.start_date))
                out.append(u"""<td style="border:0">%s</td>""" % F.NA(rg.end_date))
                out.append(u"""<td style="border:0">%s</td>""" % F.NA(rg.institution))
                out.append(u"""<td style="border:0">%s</td>""" % F.NA(rg.type))
                out.append(u"""<td style="border:0">%s</td>""" % F.NA(rg.category))
                out.append(u"</tr>")
                out.append(u"""<tr><td colspan="5">Zielsetzung: %s</tr>""" % F.NA(rg.goal))
        out.append(u"</table>")
        out.append(u"""<div class="legend">
        Legende: <strong>(BB)</strong>: Allgemein bildenden Bereich <strong>(BV)</strong>: Bereich der Berufsvorbereitung <strong>(BQ)</strong>: Berufliche Qualifizierung <strong>(LB)</strong>: Lebensbewältigung</div>""")
        return u"\n".join(out)

    def _substitute(self, client, agency, list):
        # Substitution dictionary
        s = {
                'VORNAME': client.first_name,
                'NACHNAME': client.last_name,
                'ANGEBOTE': list 
            }
        result = self.template.safe_substitute(s)
        return result 

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8:
